#!/bin/sh

echo " "
echo "NI-VISA : Add Permissions for a Raw USB Device"
echo " "
echo "This Script gives permission to all users to access a specific raw USB "
echo "device using NI-VISA.  Under the default configuration of most Linux"
echo "distributions, only the root user can access all of the USB devices."
echo "Running this script is not necessary if NI-VISA will be run only by"
echo "the 'root' user or if usbfs (formerly known as usbdevfs) is mounted"
echo "with the \"devmode=0666\" option."
echo " "

if [ -d "/etc/udev/rules.d" ] ; then
   if [ -x /sbin/udevadm ]; then
      udevCommand="/sbin/udevadm"
      udevCommandStop="$udevCommand control --stop-exec-queue"
      udevCommandStart="$udevCommand control --start-exec-queue"
   else
      echo "Cannot find \"/sbin/udevadm\"."
      echo "Please make sure that the \"udev\" package is "
      echo "installed properly on your system."
      echo "udev: http://www.kernel.org/pub/linux/utils/kernel/hotplug/udev.html"
      echo "Aborted."
      exit 1
   fi
   echo "\"udev\" detected."
   echo " "
else
   echo "Cannot find either \"/etc/udev/rules.d\"."
   echo "Please make sure that either the \"udev\" package is "
   echo "installed on your system."
   echo "udev: http://www.kernel.org/pub/linux/utils/kernel/hotplug/udev.html"
   echo "Aborted."
   exit 1
fi

if [ "`/usr/bin/whoami`" != "root" ]; then
   echo "This script needs to be run with superuser permission to install files"
   echo "on the system.  Please re-run this script as the 'root' user."
   echo "Aborted."
   exit 1
fi

NIVISA_USBRAW_VENDORID=""
NIVISA_USBRAW_PRODUCTID=""
while [ "" != "$1" ]
do
   case $(echo "$1" | cut -c -2) in
      -v)
         if [ "-v" = "$1" ]; then
            shift
            NIVISA_USBRAW_VENDORID=$1
         else
            NIVISA_USBRAW_VENDORID=$(echo "$1" | cut -c 3-)
         fi
         case "$NIVISA_USBRAW_VENDORID" in
            0[xX][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F]) ;;
            *) echo "Please enter the 4 digit hexadecimal Vendor ID number (in the form of \"0xA1B2\" or \"0x000E\")"
               exit 1
               ;;
         esac
         ;;
      -p)
         if [ "-p" = "$1" ]; then
            shift
            NIVISA_USBRAW_PRODUCTID=$1
         else
            NIVISA_USBRAW_PRODUCTID=$(echo "$1" | cut -c 3-)
         fi
         case "$NIVISA_USBRAW_PRODUCTID" in
            0[xX][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F]) ;;
            *) echo "Please enter the 4 digit hexadecimal Product ID number (in the form of \"0xA1B2\" or \"0x000E\")"
               exit 1
               ;;
         esac
         ;;
      *)
         echo "Error:"
         echo "   Unknown argument: $1"
         echo ""
         echo "Usage: $0 [options]"
         echo ""
         echo "   options:"
         echo "      -v VID   Vendor ID in the form of 0xA1B2 or 0x000E"
         echo "      -p PID   Product ID in the form of 0xA1B2 or 0x000E"
         exit 1
         ;;
   esac
   shift
done

if [ "$NIVISA_USBRAW_VENDORID" = "" ]; then
   echo "The Vendor ID (VID) of a USB device is a numeric identifier that"
   echo "may be up to 4 hexadecimal digits.  It is found in the \"device "
   echo "descriptor\" of the USB Device.  An example is \"0xA1B2\"."
   ans=""

   while [ -z $ans ]
   do
      echo " "
      echo -n "What is the Vendor ID (VID) of the USB Device? "
      read ans
      case "$ans" in
         [qQ]*) echo "Aborted."; exit 1 ;;
         0[xX][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F]) NIVISA_USBRAW_VENDORID="$ans" ;;
         *) ans="" ; echo "Please enter a 4 digit hexadecimal number (in the form of \"0xA1B2\" or \"0x000E\")" ;;
      esac
   done
   echo "The following USB Device information has been entered:"
   echo " "
   echo "   Vendor ID   (VID) :    \"${NIVISA_USBRAW_VENDORID}\""
   echo -n "Continue? [Yn] "
   read ans
   test -z "$ans" && ans=y
   case "$ans" in
      [yY]*) ;;
      *) echo "Aborted."; exit 1 ;;
   esac
   echo " "
fi

if [ "$NIVISA_USBRAW_PRODUCTID" = "" ]; then
   echo "The Product ID (PID) of a USB device is a numeric identifier that"
   echo "may be up to 4 hexadecimal digits.  It is found in the \"device "
   echo "descriptor\" of the USB Device.  An example is \"0xA1B2\"."
   ans=""

   while [ -z $ans ]
   do
      echo " "
      echo -n "What is the Product ID (PID) of the USB Device? "
      read ans
      case "$ans" in
         [qQ]*) echo "Aborted."; exit 1 ;;
         0[xX][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F]) NIVISA_USBRAW_PRODUCTID="$ans" ;;
         *) ans="" ; echo "Please enter a 4 digit hexadecimal number (in the form of \"0xC3D4\" or \"0x000F\")" ;;
      esac
   done
   echo "The following USB Device information has been entered:"
   echo " "
   echo "   Product ID  (PID) :    \"${NIVISA_USBRAW_PRODUCTID}\""
   echo -n "Continue? [Yn] "
   read ans
   test -z "$ans" && ans=y
   case "$ans" in
      [yY]*) ;;
      *) echo "Aborted."; exit 1 ;;
   esac
   echo " "
fi

echo "The following USB Device information has been entered:"
echo " "
echo "   Vendor ID   (VID) :    \"${NIVISA_USBRAW_VENDORID}\""
echo "   Product ID  (PID) :    \"${NIVISA_USBRAW_PRODUCTID}\""
echo " "

# Strip off the "0x" and convert to lower case
NIVISA_USBRAW_VENDORID_BASE=`expr substr ${NIVISA_USBRAW_VENDORID} 3 4 | tr '[A-Z]' '[a-z]'`
NIVISA_USBRAW_PRODUCTID_BASE=`expr substr ${NIVISA_USBRAW_PRODUCTID} 3 4 | tr '[A-Z]' '[a-z]'`
# Adding this line to the file "/etc/udev/rules.d/nivisa_raw.rules"
echo "SUBSYSTEM==\"usb\", ATTR{idProduct}==\"${NIVISA_USBRAW_PRODUCTID_BASE}\", ATTR{idVendor}==\"${NIVISA_USBRAW_VENDORID_BASE}\", RUN+=\"/etc/udev/agents.d/usb/nivisa_usbraw\"" >> "/etc/udev/rules.d/nivisa_usbraw.rules"
echo "SUBSYSTEM==\"usb_device\", ATTR{idProduct}==\"${NIVISA_USBRAW_PRODUCTID_BASE}\", ATTR{idVendor}==\"${NIVISA_USBRAW_VENDORID_BASE}\", RUN+=\"/etc/udev/agents.d/usb/nivisa_usbraw\"" >> "/etc/udev/rules.d/nivisa_usbraw.rules"
if [ $? != 0 ] ; then
   echo "An error occurred while trying to write to \"/etc/udev/rules.d/nivisa_usbraw.rules\"."
   echo "Aborted."
   exit 1
fi

# Ensure that udev will update the rules it uses
$udevCommandStop
$udevCommandStart

echo "If the USB Device is currently plugged in, please unplug and plug "
echo "it back in for these changes to take effect. "
echo " "
echo "USB Device successfully added."

exit 0

